/*
* Copyright (C) 2013 - 2016  Xilinx, Inc.  All rights reserved.
*
* Permission is hereby granted, free of charge, to any person
* obtaining a copy of this software and associated documentation
* files (the "Software"), to deal in the Software without restriction,
* including without limitation the rights to use, copy, modify, merge,
* publish, distribute, sublicense, and/or sell copies of the Software,
* and to permit persons to whom the Software is furnished to do so,
* subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included
* in all copies or substantial portions of the Software.
*
* Use of the Software is limited solely to applications:
* (a) running on a Xilinx device, or (b) that interact
* with a Xilinx device through a bus or interconnect.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
* IN NO EVENT SHALL XILINX  BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
* WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
* Except as contained in this notice, the name of the Xilinx shall not be used
* in advertising or otherwise to promote the sale, use or other dealings in this
* Software without prior written authorization from Xilinx.
*
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <stdint.h>
#include "lcd.h"
#include <stdbool.h>
#include <string.h>

int main(int argc, char *argv[])
{
	int fd;
	int c = 0;
	unsigned page_size = sysconf(_SC_PAGESIZE);  //Get  the number of bytes in a memory page, where "page" is a fixed-length block.
	char text[1024];


	uint16_t x_pos = 0;
	uint16_t y_pos = 0;
	uint16_t color = color_black; 
	bool init_flag = false; 
	bool demo_flag = false; 


	// Parse command line arguements 
	while((c = getopt(argc, argv, "i:s:x:y:d:c:h")) != -1) {
		switch(c) {
		case 'i': //init LCD to default state
			if(optarg[0] == 't'){
				init_flag = true;
			}
			else{
				init_flag = false;
			}

			break;
		case 's': //string to write
			strcpy(text, optarg);
			
			break;
		case 'x': //x possitin of string
			x_pos = (uint16_t) atoi(optarg);

			break;
		case 'y': //y possition of string
			y_pos = (uint16_t) atoi(optarg);
			
			break;
		case 'd': //run demo
			if(optarg[0] == 't'){
				demo_flag = true;
			}
			else{
				demo_flag = false;
			}

			break;
		case 'c': //color of text
			if(optarg[0] == 'r'){
				color = color_red;
			}
			else if(optarg[0] == 'g'){
				color = color_green;
			}
			else if(optarg[0] == 'w'){
				color = color_white;
			}
			else{
				color = color_black;
			}

			break;
		default:
			printf("invalid option: %c\n", (char)c);
			return -1;
		}
	}

	// Open /dev/mem file - memory
	fd = open ("/dev/mem", O_RDWR);
	if (fd < 1) {
		perror("Cannot open /dev/mem");
		exit(-1);
	}
	
	// map HW perypheries into the memory
	if ((io_reg0 = mmap(NULL, page_size, PROT_READ|PROT_WRITE, MAP_SHARED, fd, IO_REG0_ADR)) == MAP_FAILED) {
		perror("cannot map io_reg0");
		exit(-1);
	}

	*((unsigned *)(io_reg0 )) = 0xffffffff; //def state of pins
	usleep(1000);
	*((unsigned *)(io_reg0 )) &= (~ MASK_CS_W); //enable LCD

	 if(init_flag){
	 	LCD_Init();
	 }

	 if(demo_flag){

	 	//background color
		Text_Background_Color1(color_green);//Set the background color		
		Write_Dir(0X8E,0X80);//Start screen clearing (display window)
    	usleep(500000); //wait
    	Text_Background_Color1(color_red);//Set the background color		
		Write_Dir(0X8E,0X80);//Start screen clearing (display window)
    	usleep(500000); //wait
    	Text_Background_Color1(color_yellow);//Set the background color		
		Write_Dir(0X8E,0X80);//Start screen clearing (display window)
    	usleep(500000); //wait
    	Text_Background_Color1(color_brown);//Set the background color		
		Write_Dir(0X8E,0X80);//Start screen clearing (display window)
    	usleep(500000); //wait
    	Text_Background_Color1(color_white);//Set the background color		
		Write_Dir(0X8E,0X80);//Start screen clearing (display window)
    	usleep(500000); //wait

    	//Write text
    	Write_Dir(0x40,0x80);//Set the character mode
    	Text_Foreground_Color1(color_black);//Set the foreground color
    	FontWrite_Position(5,50);//Text written to the position
    	String("123456789 A B C D E F G H I J K L M N O P Q R S T U V W X Y Z abcdefghijklmnopqrstuvwxyz"); //write stirng to display
    	usleep(500000); //wait

    	//Write text
    	Write_Dir(0x40,0x80);//Set the character mode
    	Text_Foreground_Color1(color_red);//Set the foreground color
    	FontWrite_Position(5,200);//Text written to the position
    	String("123456789 A B C D E F G H I J K L M N O P Q R S T U V W X Y Z abcdefghijklmnopqrstuvwxyz"); //write stirng to display
    	usleep(500000); //wait

    	//Write text
    	Write_Dir(0x40,0x80);//Set the character mode
    	Text_Foreground_Color1(color_green);//Set the foreground color
    	FontWrite_Position(5,350);//Text written to the position
    	String("123456789 A B C D E F G H I J K L M N O P Q R S T U V W X Y Z abcdefghijklmnopqrstuvwxyz"); //write stirng to display
    	usleep(500000); //wait

    	//Write text
    	Write_Dir(0x40,0x80);//Set the character mode
    	Text_Foreground_Color1(color_yellow);//Set the foreground color
    	FontWrite_Position(500,80);//Text written to the position
    	String("ahoj"); //write stirng to display
    	usleep(30000000); //wait

    	//draw lines
		Write_Dir(0x40,0x00); // graphic mode 
	    for (int i = 0; i < 800; i++) 
	    {
	    	LCD_PutPixel(i, 300, color_black);
	    }
	    for (int i = 0; i < 480; i++)
	    {
	    	LCD_PutPixel(200, i, color_green);
	    }
	    usleep(100000); //wait
	    for (int i = 0; i < 480; i++)
	    {
	    	LCD_PutPixel(300, i, color_red);
	    }
	    for (int n = 45; n < 600; n+=26)
	    {
	    	for (int i = 0; i < 480; i++)
		    {
		    	LCD_PutPixel(n, i, color_brown);
		    }
	    }

	    usleep(3000000);
	    LCD_Init();
	 }


    Write_Dir(0x40,0x80);//Set the character mode
    Text_Foreground_Color1(color);//Set the foreground color
    FontWrite_Position(x_pos,y_pos);//Text written to the position
    String((uint8_t*)text);

    return 0;
}
